import pandas as pd
from dateutil.parser import parse


# hcquant_engine = None
class TradeHandle(object):
    def __init__(self, engine):
        if not isinstance(engine, str):
            raise TypeError("engine is like 'mysql://....'")
        self.engine = engine

    def step_trade_date(self, date, step):
        """
        交易日向前向后漂移
        """
        if step == 0:
            return date
        date = parse(date).strftime('%Y%m%d') if isinstance(date, str) else date.strftime('%Y%m%d')
        flag = '>' if step > 0 else '<'
        order = 'asc' if step > 0 else 'desc'
        sql = """
        SELECT TOP {} TRADE_DAYS FROM nWind.dbo.AShareCalendar
        where S_INFO_EXCHMARKET = 'SSE'
        and TRADE_DAYS {} '{}'
        order by TRADE_DAYS {}
        """.format(abs(step), flag, date, order)
        df = pd.read_sql(sql, self.engine)
        return df['TRADE_DAYS'].iloc[-1]

    def adjust_to_trade_date(self, base_date, adjust='last'):
        """
        调整基准日，若基准日不是交易日，则将其调整为之前最近的一个交易日
        """
        base_date = parse(base_date).strftime('%Y%m%d') if isinstance(base_date, str) else base_date.strftime('%Y%m%d')
        flag = '>=' if adjust == 'last' else '<='
        order = 'asc' if adjust == 'last' else 'desc'
        sql = """
        SELECT TOP 1 TRADE_DAYS FROM nWind.dbo.AShareCalendar
        where S_INFO_EXCHMARKET = 'SSE'
        and TRADE_DAYS {} '{}'
        order by TRADE_DAYS {}
        """.format(flag, base_date, order)
        df = pd.read_sql(sql, self.engine)
        return df['TRADE_DAYS'].iloc[-1]

    def get_time_list(self, start_date, end_date):
        """
        给定开始日期和结束日期，返回其中的所有交易日
        Args:
            start_date (str or datetime): 开始日期，常见日期格式支持
            end_date (str ot datetime): 结束日期，常见日期格式支持
        Returns:
            list: str,‘%Y-%m-%d’格式的日期列表

        Examples:
            >> time_list = get_time_list('20150101','20160101')
        """
        start_date = parse(start_date).strftime('%Y%m%d') if isinstance(start_date, str) else start_date.strftime('%Y%m%d')
        end_date = parse(end_date).strftime('%Y%m%d') if isinstance(end_date, str) else end_date.strftime('%Y%m%d')
        sql = """
        SELECT TRADE_DAYS FROM nWind.dbo.AShareCalendar
        where S_INFO_EXCHMARKET = 'SSE'
        and TRADE_DAYS >= '{}' 
        and TRADE_DAYS <= '{}' 
        """.format(start_date, end_date)
        df = pd.read_sql(sql, self.engine)
        return df['TRADE_DAYS'].tolist()



